/////////////////////////////////////////////////////////////////////////////
// File:            gnFileSource.h
// Purpose:         Implements generic gnBaseSource methods
// Description:     Provides a general implementation for accessing DNA
//                  sequence contig data files.
// Changes:        
// Version:         libGenome 0.1.0 
// Author:          Aaron Darling 
// Last Edited:     April 15, 2001, 11:13:00pm 
// Modified by:     
// Copyright:       (c) Aaron Darling 
// Licenses:        Proprietary 
/////////////////////////////////////////////////////////////////////////////
#include "gn/gnFileSource.h"
#include <fstream>
#include "gn/gnFilter.h"

//copy constructor
gnFileSource::gnFileSource(const gnFileSource& gnfs){
	m_openString = gnfs.m_openString;
	m_pFilter = gnfs.m_pFilter;
	m_newlineType = gnfs.m_newlineType;
	m_newlineSize = gnfs.m_newlineSize;
	m_ifstream.open( m_openString.c_str(), ios::in | ios::binary );
	if( !m_ifstream.is_open() )
		m_ifstream.clear();
}

// Open, Close	
void gnFileSource::Open( string openString )
{
	m_ifstream.open(openString.c_str(), ios::in | ios::binary );
	if( m_ifstream.is_open() )
	{
		m_openString = openString;
		if( ParseStream(m_ifstream) )
		{
			;
		}
		else{
			m_ifstream.clear();
			m_ifstream.close();
		}
	}else{
		m_ifstream.clear();
		Throw_gnEx(FileNotOpened());
	}
}
void gnFileSource::Open( )
{
	m_ifstream.open( m_openString.c_str(), ios::in | ios::binary );
	if( !m_ifstream.is_open() ){
		m_ifstream.clear();
		Throw_gnEx(FileNotOpened());
	}
}
void gnFileSource::Close()
{
	m_ifstream.close();
	if( m_ifstream.is_open() )
		Throw_gnEx(IOStreamFailed());
}

boolean gnFileSource::Read( const uint64 pos, char* buf, uint32& bufLen) 
{
	m_ifstream.seekg(pos, ios::beg);
	m_ifstream.read(buf, bufLen);
	if(m_ifstream.fail()){
		m_ifstream.clear();
		return false;
	}
	return true;
}
